import { useState, useEffect, useCallback, useMemo } from 'react'

import { AxiosResponse } from 'axios'
import { useAppDispatch } from 'stores'
import { loadingActions } from 'stores/loading'
import { ApiStatus } from 'types'

import { API_STATUS } from '../constants'

type Options = {
  // call function khi khởi tạo hay không ?
  immediate?: boolean // default = true
  loadingGlobal?: boolean // default = true
}

export const useApi = <T, E = string>(
  asyncFunction: () => Promise<AxiosResponse<T>>,
  options?: Options
) => {
  const [status, setStatus] = useState<ApiStatus>(API_STATUS.IDLE)
  const [data, setData] = useState<T | null>(null)
  const [error, setError] = useState<E | null>(null)
  const [isLoading, setIsLoading] = useState<boolean>(false)
  const [isError, setIsError] = useState<boolean>(false)
  const [isSuccess, setIsSuccess] = useState<boolean>(false)

  const newOptions = useMemo<Options>(
    () => ({
      immediate: true,
      loading: true,
      ...options,
    }),
    [options]
  )

  const dispatch = useAppDispatch()

  const execute = useCallback(() => {
    // Nếu config loading = false => không hiển thị Loading
    if (newOptions.loadingGlobal) {
      dispatch(loadingActions.showLoading())
    }
    setStatus(API_STATUS.PENDING)
    setData(null)
    setError(null)
    setIsLoading(true)
    setIsSuccess(false)
    setIsError(false)

    return asyncFunction()
      .then((response: AxiosResponse<T>) => {
        setData(response.data)
        setStatus(API_STATUS.SUCCESS)
        setIsSuccess(true)
        return response.data
      })
      .catch((error: any) => {
        setIsError(true)
        setError(error)
        setStatus(API_STATUS.ERROR)
        // handle error
        // ...
      })
      .finally(() => {
        setIsLoading(false)
        if (newOptions.loadingGlobal) {
          dispatch(loadingActions.hideLoading())
        }
      })
  }, [asyncFunction])

  useEffect(() => {
    if (newOptions.immediate) {
      execute()
    }
  }, [newOptions.immediate])

  return { execute, status, data, error, isLoading, isError, isSuccess }
}
